/***************************************************************************
                          cdp.h  -  description
                             -------------------
    begin                : Sun Aug 26 2001
    copyright            : (C) 2001, 2002 by Thomas Friedrichsmeier
    email                : Thomas.Friedrichsmeier@ruhr-uni-bochum.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CDP_H
#define CDP_H

/**Common debugging and prefixing class.
Takes care of file-operations that are needed in several of the other classes, most prominently of
prefixing paths and checking for existance of files. Also covers the debugging functionality.
  *@author Thomas Friedrichsmeier
  */

#include <qobject.h>
#include <qmap.h>

#include "tpbase.h"

class QString;
class QDomElement;
class QDomNode;
class QDomNodeList;
class QWidget;
class QCanvasPixmap;
class QCanvasPixmapArray;

class Cdp : public TPBase {
  public:
	Cdp();
	~Cdp();
/** Initializes the function that will take care of debugging messages and potential errors. in_file
is the filename which is currently to be checked. */
	void init_debug(const QString & in_file);
/** Appends an arbitrary string to the debug or error messages. If level == Critical, this is a critical
error and the program will abort in do_debug (). If *in_node is specified a backtrace is shown */
	void debug_msg(const QString & message, DebugLevel level, QDomNode const *in_node = 0);
/** Add a directory prefix to a filename if appropriate (i.e. not an absolute path) and checks,
whether the file exists. type specifies, whether this is an image, a sound or a plain conf-file
(different prefixes will be used). Level specifies the error-level if file could not be opened for reading */
	QString prefix_check(const QString & filename, FileType type, DebugLevel level);
/** prefix of the resource-dir, where the configuration-files are to be found */
	QString data_prefix;
/** Alternative prefix to the configuration-files */
	QString alt_data_prefix;
/** prefix of the resource-dir, where the image-files are to be found */
	QString image_prefix;
/** alternative prefix to the image-files */
	QString alt_image_prefix;
/** prefix of the resource-dir, where the sound-files are to be found */
	QString sound_prefix;
/** alternative prefix to the sound-files */
	QString alt_sound_prefix;
/** Checks for the existence of a certain integer XML-attribute in element. If given, returns its
value. If not given or not an int or not between min and max, return def and writes debugging message of level level */
	int get_int_attribute(const QString & attribute, const QDomElement & element, int min, int max, int def, DebugLevel level);
/** Checks for the existence of a certain float XML-attribute in element. If given, returns its
value. If not given or not a float number or not between min and max, returns def and writes debugging message of level level */
	float get_float_attribute(const QString & attribute, const QDomElement & element, float min, float max, float def,
								DebugLevel level);
/** Checks for the existence of a certain string XML-attribute in element. If given, returns its
value. If not given or not existent, returns def and writes debugging message of level level */
	QString get_string_attribute(const QString & attribute, const QDomElement & element, const QString & def, DebugLevel level, bool trans=true);
/** Checks for the existence of a certain bool XML-attribute in element. If given, returns its
value. If not given or not existent, returns def and writes debugging message of level level */
	bool get_bool_attribute(const QString & attribute, const QDomElement & element, bool def, DebugLevel level);
/** Opens the config-file file, parses it into XML, and returns document element */
	QDomElement open_config_file(const QString &file);
/** Gets all child-nodes named name of element parent, checking whether there are at least
min, at most max nodes and min_if_any nodes if any, otherwise stating an error at
level level. (if name = "", don't check for name, but return all child nodes, if max < 0, there is no maximum) */
	QDomNodeList get_node_list(const QString & name, const QDomElement & parent, int min, int max, int min_if_any,
							   DebugLevel level);
/** Get the child-element named name of element parent. If no such element exists
	or there are more than one element, outputs error at level level. */
	QDomElement get_element(const QString & name, const QDomElement & parent, DebugLevel level);
/** Returns inverse of errors_occurred, displaying a Dialog if errors_occured */
	bool ok();
/** Checks for the existence of a certain string XML-attribute in element. If given, checks
whether the attribute's value matches any of allowed_values, which is a ';' seperated
list of allowed strings. If not given, not valid or not existent, returns def and writes
debugging message of level level. */
	QString get_multi_choice_attribute(const QString & attribute, const QDomElement & element, const QString & allowed_values, const QString & def, DebugLevel level);
/** Returns true, if the commandline-option highlight was set, false otherwise */
	bool draw_highlight ();
/** Returns true, if the commandline-option thorough_check was set, false otherwise */
	bool thorough_check();
/** Returns whether we're reading for real (not just for the credits/startscreen) */
	bool for_real ();
/** Sets whether reading is going to be for real */
	void set_for_real (bool real);
/** Return the default mission path */
	QString defaultPath ();
/** Return the startup-Mission */
	QString startupMission ();
/** Return calculations-per-second setting */
	int cps ();
/** Return frames-per-second setting */
	int fps ();
/** Signals, that the mission mission has just been opened (and therefore is the one
last chosen) */
	void setLastLoaded (const QString &mission);
/** Saves the current config (and emits a config_changed ()) */
	void set_new_config ();
/** is sound enabled? */
	bool sound_enabled ();
/** Does everything from reading a filename for an image from the given element->attribute over prefixing to creating the image as a QCanvasPixmap. In case creation fails, an empty (black) Pixmap of width def_width and height def_height is returned. Therefore you can definitely safely use the returned Pixmap, even if loading failed. */
	QCanvasPixmap * createQCPixmap (const QString & attribute, const QDomElement & element, int def_width, int def_height, DebugLevel level);
/** See above, but reads image from the already known (but not necessarily prefixed) filename filename. */
	QCanvasPixmap * createQCPixmap (const QString & filename, int def_width, int def_height, DebugLevel level);
/** Similar to createQCPixmap. Reads a filename(-pattern) from element->attribute and creates a QCanvasPixmapArray
(with at most frames frames) accordingly. If loading the images fails, returns a PixmapArray with a single (black) frame
of width def_widht and height def_height + sets frames to 1. */
	QCanvasPixmapArray * createQCPixmapArray (const QString & attribute, const QDomElement & element, int * frames, int def_width, int def_height, DebugLevel level);
/** Creates a single frame QCanvasPixmapArray with the text text_p written in font font (given as it's name) and using color color.
Qt::TextFlags can be used for formatting. The text will be written in a font-size such that it will completely fit inside an area of
max_widht / height, trying to get as close to max_width / height as possible. If smooth is specified, only font-sizes that really are
available on the system (without scaling) will be used. Note that on some systems this is always the case, and the option might
not have any effect. clip and align_handle jointly specify where the hot spot of the returned frame will be. If align_handle is
given, the hot spot will be placed according to the text_flags-alignment (e.g. in the center of the image for center-align text). If align_handle
is not given, the top-left will always be used. In this case, if clip is false, the top-left of a hypothetic, unclipped image of max_width / height
will be used, whereas is clip is true, the natural top-left of the clipped image is used. Technically the image will always be clipped
to the dimensions actually needed in order to save resources, but with clip set to false, it will seem like it had not been clipped. */
	QCanvasPixmapArray * createTextPixmapArray (const QString & text_p, const QString & font, QColor color, int text_flags, int max_width, int max_height, bool smooth, bool clip, bool align_handle);
/** See above, but reads the parameters from QDomElement e */
	QCanvasPixmapArray * createTextPixmapArray (const QDomElement & element);

  private:
	bool draw_highlights;
	bool opt_thorough_check;
	bool read_for_real;
/** The filenmae of the currently debugged config file */
	 QString debug_in_file;
/** The debug_level */
	int debug_level;
/** Traces back the nodes in the current file, so as to display them in the debugging-output */
	QString backtrace(const QDomNode & node);
/** Makes sense only when die_on_error == false. Keeps track of critical errors*/
	bool errors_occurred;
/** Stores translation-table for the config-file currently opened */
	QMap <QString, QString> translations;

friend class Preferences;

/** Preferences */
	QString pref_DefaultMission;
/** Preferences */
	QString pref_LastMission;
/** Preferences */
	QString pref_SelectedMission;
/** Preferences */
	bool pref_useDefaultMission;	
/** Preferences */
	bool pref_useLastMission;
/** Preferences */
	bool pref_useSelectedMission;
/** Preferences */
	int pref_fps;
/** Preferences */
	int pref_cps;
/** Preferences */
	bool pref_sound;
};

#endif
